@@ -576,6 +576,14 @@ class VipsBand(Band):
 
         self._band_no = band_no
 
+    @property
+    def XSize(self):
+        return self._dataset.RasterXSize
+
+    @property
+    def YSize(self):
+        return self._dataset.RasterYSize
+
     def ReadAsArray(self, xoff=0, yoff=0, win_xsize=None, win_ysize=None,
                     buf_xsize=None, buf_ysize=None, buf_obj=None):
         """
@@ -599,8 +607,8 @@ class VipsBand(Band):
                                  width=win_xsize, height=win_ysize)
 
         return numpy.ndarray(shape=(win_xsize, win_ysize),
-                             buffer=area.tobuffer(),
-                             dtype=band.NumPyType())
+                             buffer=bytes(area.tobuffer()),
+                             dtype=band.NumPyType()).copy()
 
     # The next methods are there to prevent you from shooting yourself in the
     # foot.
@@ -656,9 +664,16 @@ class VipsDataset(Dataset):
         datatype = self.GetRasterBand(1).NumPyDataType
 
         return numpy.ndarray(shape=(self.RasterCount, ysize, xsize),
-                             buffer=area.tobuffer(),
+                             buffer=bytes(area.tobuffer()),
                              dtype=datatype)
 
+    def colorize(self, colors):
+        """Replaces this image with a colorized version."""
+        with LibVips.disable_warnings():
+            nodata = self.GetRasterBand(1).GetNoDataValue()
+            self._image = colors.colorize(image=self.image, nodata=nodata)
+
+
     def _upsample(self, ratios):
         if ratios == XY(x=1.0, y=1.0):
             # No upsampling needed
@@ -731,8 +746,27 @@ class VipsDataset(Dataset):
              pixel_sizes.y)
         ))
 
-        width = int(tile_extents.dimensions.x / pixel_sizes.x)
-        height = int(tile_extents.dimensions.y / pixel_sizes.y)
+        # Defining
+        epsilon_lower = 1e-4
+        epsilon_higher = 1 - epsilon_lower
+
+        # Verifying that the width and height are within an acceptable
+        # floating point error.
+        width = tile_extents.dimensions.x / pixel_sizes.x
+        height = tile_extents.dimensions.y / pixel_sizes.y
+        if epsilon_lower < (width % 1) < epsilon_higher:
+            raise AssertionError(
+                'width {0!r} is not within an acceptable range of '
+                'an integer'.format(width)
+            )
+        if epsilon_lower < (height % 1) < epsilon_higher:
+            raise AssertionError(
+                'height {0!r} is not within an acceptable range of '
+                'an integer'.format(height)
+            )
+
+        width = int(round(width))
+        height = int(round(height))
 
         if left == top == 0 and \
            width == self.RasterXSize and \
@@ -1000,14 +1034,11 @@ class TmsPyramid(object):
         self.max_resolution = max_resolution
 
         self._dataset = None
-        self._image = None
         self._resolution = None
 
     def colorize(self, colors):
         """Replaces this image with a colorized version."""
-        with LibVips.disable_warnings():
-            nodata = self.dataset.GetRasterBand(1).GetNoDataValue()
-            self._image = colors.colorize(image=self.image, nodata=nodata)
+        return self.dataset.colorize(colors)
 
     @property
     def dataset(self):
